@@ -1,16 +1,16 @@
-# $Id: //depot/ldap/lib/Net/LDAP/Message.pm#17 $
+# $Id: //depot/ldap/lib/Net/LDAP/Message.pm#19 $
 # Copyright (c) 1997-2000 Graham Barr <gbarr@pobox.com>. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 
 package Net::LDAP::Message;
 
-use Net::LDAP::Constant qw(LDAP_SUCCESS);
+use Net::LDAP::Constant qw(LDAP_SUCCESS LDAP_COMPARE_TRUE LDAP_COMPARE_FALSE);
 use Net::LDAP::ASN qw(LDAPRequest);
 use strict;
 use vars qw($VERSION);
 
-$VERSION = "1.03";
+$VERSION = "1.05";
 
 my $MsgID = 0;
 
@@ -71,7 +71,7 @@ sub referrals {
     : ();
 }
 
-sub error {
+sub server_error {
   my $self = shift;
 
   $self->sync unless exists $self->{resultCode};
@@ -81,6 +81,13 @@ sub error {
     : undef
 }
 
+sub error {
+  my $self = shift;
+  $self->server_error
+    or require Net::LDAP::Util
+    and Net::LDAP::Util::ldap_error_desc( $self->code );
+}
+
 sub set_error {
   my $self = shift;
   ($self->{resultCode},$self->{errorMessage}) = ($_[0]+0, "$_[1]");
@@ -182,6 +189,7 @@ sub pdu      {  shift->{pdu}      }
 sub callback {  shift->{callback} }
 sub parent   {  shift->{parent}   }
 sub mesg_id  {  shift->{mesgid}   }
+sub is_error {  shift->code       }
 
 ##
 ##
@@ -196,6 +204,12 @@ sub mesg_id  {  shift->{mesgid}   }
 @Net::LDAP::Unbind::ISA  = qw(Net::LDAP::Message::Dummy);
 @Net::LDAP::Abandon::ISA = qw(Net::LDAP::Message::Dummy);
 
+sub Net::LDAP::Compare::is_error {
+  my $mesg = shift;
+  my $code = $mesg->code;
+  $code != LDAP_COMPARE_FALSE and $code != LDAP_COMPARE_TRUE
+}
+
 {
   package Net::LDAP::Message::Dummy;
   use vars qw(@ISA);
